defineVirtualDevice("controllerData", {
    title: "Controller Data",
    cells: {
        "CPU Utilization (prcnt)": {
            type: "range",
            min: 0,
            max: 100,
            value: 0,
            forceDefault: true,
            order: 1,
        },
        "RAM Total MB": {
            type: "value",
            value: 0,
            forceDefault: true,
            order: 2,
        },
        "RAM Available MB": {
            type: "value",
            value: 0,
            forceDefault: true,
            order: 3,
        },
        "RAM Used (prcnt)": {
            type: "range",
            min: 0,
            max: 100,
            value: 0,
            forceDefault: true,
            order: 4,
        },
        "RootFS Total MB": {
            type: "value",
            value: 0,
            forceDefault: true,
            order: 5,
        },
        "RootFS Free MB": {
            type: "value",
            value: 0,
            forceDefault: true,
            order: 6,
        },
        "RootFS Used (prcnt)": {
            type: "range",
            min: 0,
            max: 100,
            value: 0,
            forceDefault: true,
            order: 7,
        },
        "Mnt Data Total MB": {
            type: "value",
            value: 0,
            forceDefault: true,
            order: 8,
        },
        "Mnt Data Free MB": {
            type: "value",
            value: 0,
            forceDefault: true,
            order: 9,
        },
        "Mnt Data Used (prcnt)": {
            type: "range",
            min: 0,
            max: 100,
            value: 0,
            forceDefault: true,
            order: 10,
        },
    },
});

var prev_total = 0;
var prev_idle = 0;

function updateCpuUtilization() {
    runShellCommand("grep 'cpu ' /proc/stat", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            if (exitCode == 0) {
                var splittedOutput = capturedOutput.split(" ");
                if (splittedOutput != null && splittedOutput.length > 0) {
                    var idle = parseInt(splittedOutput[5]);
                    var total = 0;
                    for (i = 2; i <= splittedOutput.length - 1; i++) {
                        total += parseInt(splittedOutput[i]);
                    }
                    var utilization = (
                        (1 - (idle - prev_idle) / (total - prev_total)) *
                        100
                    ).toFixed();
                    dev["controllerData"]["CPU Utilization (prcnt)"] = parseInt(utilization);
                    prev_idle = isNaN(idle) ? 0 : idle;
                    prev_total = isNaN(total) ? 0 : total;
                }
            }
        },
    });
}

function updateRamUtilization() {
    runShellCommand("grep MemTotal /proc/meminfo | awk '{print $2}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            if (exitCode == 0) {
                dev["controllerData"]["RAM Total MB"] = Math.round(parseInt(capturedOutput) / 1024);
            }
        },
    });

    runShellCommand("grep MemAvailable /proc/meminfo | awk '{print $2}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            if (exitCode == 0) {
                dev["controllerData"]["RAM Available MB"] = Math.round(
                    parseInt(capturedOutput) / 1024
                );
            }
        },
    });

    dev["controllerData"]["RAM Used (prcnt)"] = Math.round(
        ((dev["controllerData/RAM Total MB"] - dev["controllerData"]["RAM Available MB"]) /
            dev["controllerData/RAM Total MB"]) *
            100
    );
}

function updateDiskUtilization() {
    runShellCommand("df -m / | tail -1 | awk '{print $2}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            dev["controllerData"]["RootFS Total MB"] = parseInt(capturedOutput);
        },
    });
    runShellCommand("df -m / | tail -1 | awk '{print $4}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            dev["controllerData"]["RootFS Free MB"] = parseInt(capturedOutput);
        },
    });
    runShellCommand("df -m / | tail -1 | awk '{print $5}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            dev["controllerData"]["RootFS Used (prcnt)"] = parseInt(capturedOutput);
        },
    });

    runShellCommand("df -m /mnt/data | tail -1 | awk '{print $2}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            dev["controllerData"]["Mnt Data Total MB"] = parseInt(capturedOutput);
        },
    });
    runShellCommand("df -m /mnt/data | tail -1 | awk '{print $4}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            dev["controllerData"]["Mnt Data Free MB"] = parseInt(capturedOutput);
        },
    });
    runShellCommand("df -m /mnt/data | tail -1 | awk '{print $5}'", {
        captureOutput: true,
        exitCallback: function (exitCode, capturedOutput) {
            dev["controllerData"]["Mnt Data Used (prcnt)"] = parseInt(capturedOutput);
        },
    });
}

function fastUpdate() {
    updateCpuUtilization();
    updateRamUtilization();
}

function slowUpdate() {
    updateDiskUtilization();
}

fastUpdate();
slowUpdate();

setInterval(fastUpdate, 2000);
setInterval(slowUpdate, 10000);
